﻿using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Infrastructure;
using Microsoft.Extensions.Configuration;
using System;
using System.IO;

namespace Benday.EfCore.Api
{
    public class MyDbContextFactory : IDbContextFactory<MyDbContext>
    {
        public MyDbContext Create()
        {
            Console.WriteLine("**** MyDbContextFactory.Create()");

            var environmentName =
                        Environment.GetEnvironmentVariable(
                            "Hosting:Environment");

            Console.WriteLine("**** HOSTING ENVIRONNMENT: '{0}'", environmentName);

            var basePath = AppContext.BaseDirectory;

            return Create(basePath, environmentName);
        }

        public MyDbContext Create(DbContextFactoryOptions options)
        {
            Console.WriteLine("**** MyDbContextFactory.Create(DbContextFactoryOptions)");
            
            return Create(
                options.ContentRootPath,
                options.EnvironmentName);
        }

        void WriteConfigToConsole(IConfigurationRoot config)
        {
            var envVars = Environment.GetEnvironmentVariables();

            Console.WriteLine("*** ENVIRONMENT VARIABLES -- START ***");

            foreach (var key in envVars.Keys)
            {
                Console.WriteLine("{0}: {1}", key, envVars[key]);
            }

            Console.WriteLine("*** ENVIRONMENT VARIABLES -- END ***");

            Console.WriteLine("*** CONFIG VARIABLES -- START ***");
            
            foreach (var item in config.AsEnumerable())
            {
                Console.WriteLine("{0}: {1}", item.Key, item.Value);                
            }

            Console.WriteLine("*** CONFIG VARIABLES -- END ***");
        }

        private MyDbContext Create(string basePath, string environmentName)
        {
            Console.WriteLine(
                "**** Environment name: '{0}' ****", 
                environmentName);

            var builder = new Microsoft.Extensions.Configuration.ConfigurationBuilder()
                .SetBasePath(basePath)
                .AddJsonFile("appsettings.json");
                //.AddJsonFile($"appsettings.{environmentName}.json", true)
                //.AddEnvironmentVariables();

            var config = builder.Build();

            WriteConfigToConsole(config);
            
            var connstr = config.GetConnectionString("default");

            Console.WriteLine("**** CONN STRING: {0} ****", connstr);

            if (String.IsNullOrWhiteSpace(connstr) == true)
            {
                throw new InvalidOperationException(
                    "Could not find a connection string named 'default'.");
            }
            else
            {
                return Create(connstr);
            }
        }

        private MyDbContext Create(string connectionString)
        {
            if (string.IsNullOrEmpty(connectionString))
                throw new ArgumentException(
                    $"{nameof(connectionString)} is null or empty.",
                    nameof(connectionString));

            var optionsBuilder =
                new DbContextOptionsBuilder<MyDbContext>();

            optionsBuilder.UseSqlServer(connectionString);

            return new MyDbContext(optionsBuilder.Options);
        }
    }
}




